/*
 * (C) Copyright 2001
 * Josh Huber <huber@mclx.com>, Mission Critical Linux, Inc.
 *
 * modified for marvell db64360 eval board by
 * Ingo Assmus <ingo.assmus@keymile.com>
 *
 * modified for Tundra tsi108emu and tsi108svb boards by
 * Alex Bounine <alexandreb@tundra.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * serial.c - serial support for the Tsi108 Emulation board
 *            uses standard 16550 UART driver for both channels 
 */


#include <common.h>
#include <ns16550.h>

const NS16550_t COM_PORTS[] = { (NS16550_t) (CFG_DUART_IO + 0),
	 (NS16550_t) (CFG_DUART_IO + 0x400)
};

int serial_init (void)
{
	DECLARE_GLOBAL_DATA_PTR;
	int clock_divisor = (CFG_OCN_CLK/2) / gd->baudrate;

	NS16550_init(COM_PORTS[0], clock_divisor);
	NS16550_init(COM_PORTS[1], clock_divisor);
	return (0);
}

void serial_putc (const char c)
{
	if (c == '\n')
		NS16550_putc (COM_PORTS[CFG_DUART_CHAN], '\r');

	NS16550_putc (COM_PORTS[CFG_DUART_CHAN], c);
}

int serial_getc (void)
{ 
	return NS16550_getc (COM_PORTS[CFG_DUART_CHAN]);
}

int serial_tstc (void)
{
	return NS16550_tstc (COM_PORTS[CFG_DUART_CHAN]);
}

void serial_setbrg (void)
{
	DECLARE_GLOBAL_DATA_PTR;
	int clock_divisor = (CFG_OCN_CLK/2) / gd->baudrate;

	NS16550_reinit (COM_PORTS[0], clock_divisor);
	NS16550_reinit (COM_PORTS[1], clock_divisor);
}

void serial_puts (const char *s)
{
	while (*s) {
		serial_putc (*s++);
	}
}

#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
void kgdb_serial_init (void)
{
}

void putDebugChar (int c)
{
#ifdef CFG_KGDB_USE_OTHER_PORT
	if (c == '\n')
		NS16550_putc (COM_PORTS[(1-CFG_DUART_CHAN)], '\r');

	NS16550_putc (COM_PORTS[(1-CFG_DUART_CHAN)], c);
#else
	serial_putc (c);
#endif
}

void putDebugStr (const char *str)
{
#ifdef CFG_KGDB_USE_OTHER_PORT
	while (*str) {
		putDebugChar (*str++);
	}
#else
	serial_puts (str);
#endif
}

int getDebugChar (void)
{
#ifdef CFG_KGDB_USE_OTHER_PORT
	return NS16550_getc (COM_PORTS[(1-CFG_DUART_CHAN)]);
#else
	return serial_getc ();
#endif
}

void kgdb_interruptible (int yes)
{
	return;
}
#endif /* CFG_CMD_KGDB */
