/*
 * (C) Copyright 2005 Tundra Semiconductor Corp.
 * Alex Bounine, <alexandreb@tundra.com).
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * tsi108.h - (minimal set of) definitions used for support and
 *            initialization of Tsi108 host bridge.
 */

#ifndef _TSI108_H_
#define _TSI108_H_

#define TSI108_HLP_REG_OFFSET (0x0000)
#define TSI108_PCI_REG_OFFSET (0x1000)
#define TSI108_CLK_REG_OFFSET (0x2000)
#define TSI108_PB_REG_OFFSET  (0x3000)
#define TSI108_SD_REG_OFFSET  (0x4000)
#define TSI108_MPIC_REG_OFFSET  (0x7400)


#define PB_ID               (0x000)
#define PB_RSR              (0x004)
#define PB_BUS_MS_SELECT    (0x008)
#define PB_ISR              (0x00C)
#define PB_ARB_CTRL         (0x018)
#define PB_PVT_CTRL2		(0x034)
#define PB_SCR              (0x400)
#define PB_ERRCS            (0x404)
#define PB_AERR             (0x408)
#define PB_REG_BAR          (0x410)
#define PB_OCN_BAR1         (0x414)
#define PB_OCN_BAR2         (0x418)
#define PB_SDRAM_BAR1       (0x41C)
#define PB_SDRAM_BAR2       (0x420)
#define PB_MCR              (0xC00)
#define PB_MCMD             (0xC04)

#define PCI_CSR             (0x004)
#define PCI_P2O_BAR0        (0x010)
#define PCI_P2O_BAR0_UPPER  (0x014)
#define PCI_P2O_BAR2        (0x018)
#define PCI_P2O_BAR2_UPPER  (0x01C)
#define PCI_P2O_BAR3        (0x020)
#define PCI_P2O_BAR3_UPPER  (0x024)

#define PCI_MISC_CSR        (0x040)
#define PCI_P2O_PAGE_SIZES  (0x04C)

#define PCI_PCIX_STAT       (0x0F4)

#define PCI_IRP_STAT        (0x184)

#define PCI_PFAB_BAR0       (0x204)
#define PCI_PFAB_BAR0_UPPER (0x208)
#define PCI_PFAB_IO         (0x20C)
#define PCI_PFAB_IO_UPPER   (0x210)

#define PCI_PFAB_MEM32          (0x214)
#define PCI_PFAB_MEM32_REMAP    (0x218)
#define PCI_PFAB_MEM32_MASK     (0x21C)

#define CG_PLL0_CTRL0       (0x210)
#define CG_PLL0_CTRL1       (0x214)
#define CG_PLL1_CTRL0       (0x220)
#define CG_PLL1_CTRL1       (0x224)
#define CG_PWRUP_STATUS     (0x234)

#define MPIC_CSR(n) (0x30C + (n * 0x40))   

#define SD_CTRL         (0x000)
#define SD_STATUS       (0x004)
#define SD_TIMING       (0x008)
#define SD_REFRESH      (0x00C)
#define SD_INT_STATUS   (0x010)
#define SD_INT_ENABLE   (0x014)
#define SD_INT_SET      (0x018)
#define SD_D0_CTRL      (0x020)
#define SD_D1_CTRL      (0x024)
#define SD_D0_BAR       (0x028)
#define SD_D1_BAR       (0x02C)
#define SD_ECC_CTRL     (0x040)
#define SD_DLL_STATUS   (0x250)

#define TS_SD_CTRL_ENABLE  (1 << 31)

#define PB_ERRCS_ES         (1 << 1)
#define PB_ISR_PBS_RD_ERR   (1 << 8)
#define PB_PVT_CTRL_DSBL	(1 << 0)
#define PB_ARB_CTRL_PARK	(1 << 0)
#define PB_ARB_CTRL_BM_PARK	(3 << 1)
#define PB_ARB_CTRL_M1_EN	(1 << 9)
#define PCI_IRP_STAT_P_CSR  (1 << 23)

#endif /* _TSI108_H_ */
