/*
 * (C) Copyright 2005 Tundra Semiconductor Corp.
 * Alex Bounine, <alexandreb@tundra.com).
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * tsi108_init.c - tsi108 initialization code.
 */

#include <common.h>
#include <74xx_7xx.h>
#include "tsi108.h"

//extern void mpicInit(int verbose);

/*===========================================================================
 * Processor bus Look-up Tables Initialization Values
 */

typedef struct {
    ulong upper;
    ulong lower;
} PB2OCN_LUT_ENTRY;

static PB2OCN_LUT_ENTRY pb2ocn_lut1[32] =
{
    /* LUT entries 0 - 7 */
    {0x00000000, 0x00000201},  /* PBA=0xE000_0000 -> PCI/X (Byte-Swap) */
    {0x00000000, 0x00000201},  /* PBA=0xE100_0000 -> PCI/X (Byte-Swap) */
    {0x00000000, 0x00000201},  /* PBA=0xE200_0000 -> PCI/X (Byte-Swap) */
    {0x00000000, 0x00000201},  /* PBA=0xE300_0000 -> PCI/X (Byte-Swap) */
    {0x00000000, 0x00000201},  /* PBA=0xE400_0000 -> PCI/X (Byte-Swap) */
    {0x00000000, 0x00000201},  /* PBA=0xE500_0000 -> PCI/X (Byte-Swap) */
    {0x00000000, 0x00000201},  /* PBA=0xE600_0000 -> PCI/X (Byte-Swap) */
    {0x00000000, 0x00000201},  /* PBA=0xE700_0000 -> PCI/X (Byte-Swap) */
    /* LUT entries 8 - 15 */
    {0x00000000, 0x18000044},  /* PBA=0xE800_0000 -> SDRAM_OCN */
    {0x00000000, 0x19000044},  /* PBA=0xE900_0000 -> SDRAM_OCN */
    {0x00000000, 0x1A000044},  /* PBA=0xEA00_0000 -> SDRAM_OCN */
    {0x00000000, 0x1B000044},  /* PBA=0xEB00_0000 -> SDRAM_OCN */
    {0x00000000, 0x1C000044},  /* PBA=0xEC00_0000 -> SDRAM_OCN */
    {0x00000000, 0x1D000044},  /* PBA=0xED00_0000 -> SDRAM_OCN */
    {0x00000000, 0x1E000044},  /* PBA=0xEE00_0000 -> SDRAM_OCN */
    {0x00000000, 0x1F000044},  /* PBA=0xEF00_0000 -> SDRAM_OCN */
    /* LUT entries 16 - 23 */
    {0x00000000, 0x00000001},  /* PBA=0xF000_0000 -> PCI/X */
    {0x00000000, 0x00000001},  /* PBA=0xF100_0000 -> PCI/X */
    {0x00000000, 0x00000001},  /* PBA=0xF200_0000 -> PCI/X */
    {0x00000000, 0x00000001},  /* PBA=0xF300_0000 -> PCI/X */
    {0x00000000, 0x00000001},  /* PBA=0xF400_0000 -> PCI/X */
    {0x00000000, 0x00000001},  /* PBA=0xF500_0000 -> PCI/X */
    {0x00000000, 0x00000001},  /* PBA=0xF600_0000 -> PCI/X */
    {0x00000000, 0x00000001},  /* PBA=0xF700_0000 -> PCI/X */
    /* LUT entries 24 - 31 */
    {0x00000000, 0x00000001},  /* PBA=0xF800_0000 -> PCI/X */
    {0x00000000, 0x00000001},  /* PBA=0xF900_0000 -> PCI/X */
    {0x00000000, 0x00000001},  /* PBA=0xFA00_0000 -> PCI/X */
    {0x00000000, 0x00000001},  /* PBA=0xFB00_0000 -> PCI/X */
    {0x00000000, 0x00000001},  /* PBA=0xFC00_0000 -> PCI/X */ 
    {0x00000000, 0x00000201},  /* PBA=0xFD00_0000 -> PCI/X : (PCI/X IO Space) */
    {0x00000000, 0x00000201},  /* PBA=0xFE00_0000 -> PCI/X : (PCI/X Configuration Space: Byte-Swap) */
    {0x00000000, 0x00000240}   /* PBA=0xFF00_0000 -> HLP : (Translation Enabled + Byte-Swap) */
};

#ifdef CFG_CLK_SPREAD
typedef struct {
    ulong ctrl0;
    ulong ctrl1;
} PLL_CTRL_SET;

/*===========================================================================
 * Clock Generator SPLL0 initialization values
 */

/* PLL0 configuration table for various PB_CLKO freq.
 * Uses pre-calculated values for Fs = 30 kHz, D = 0.5%
 * Fout depends on required PB_CLKO. Based on Fref = 33 MHz
 */
static PLL_CTRL_SET pll0_config[8] =
{
    {0x00000000, 0x00000000}, /* 0: bypass */
    {0x00000000, 0x00000000}, /* 1: reserved */
    {0x00430044, 0x00000043}, /* 2: CG_PB_CLKO = 183 MHz */
    {0x005c0044, 0x00000039}, /* 3: CG_PB_CLKO = 100 MHz */
    {0x005c0044, 0x00000039}, /* 4: CG_PB_CLKO = 133 MHz */
    {0x004a0044, 0x00000040}, /* 5: CG_PB_CLKO = 167 MHz */
    {0x005c0044, 0x00000039}, /* 6: CG_PB_CLKO = 200 MHz */
    {0x004f0044, 0x0000003e}  /* 7: CG_PB_CLKO = 233 MHz */
};
#endif /* CFG_CLK_SPREAD */

/*
 * Prosessor Bus Clock (in MHz) defined by CG_PB_SELECT
 * (based on recommended Tsi108 reference clock 33MHz)
 */
static int pb_clk_sel[8] = {0,0,183,100,133,167,200,233};


/*===========================================================================
 * board_early_init_f()
 *
 * board-specific initialization executed from flash
 *
 *==========================================================================*/

int board_early_init_f (void)
{
#if !defined(CFG_BUS_CLK)
	DECLARE_GLOBAL_DATA_PTR;
    ulong i;

    /* Detect PB clock freq. */
    i = in32(CFG_TSI108_CSR_BASE + TSI108_CLK_REG_OFFSET + CG_PWRUP_STATUS);
    i = (i >> 16) & 0x07; /* Get PB PLL multiplier */
	gd->bus_clk = pb_clk_sel[i] * 1000000;
#endif /* !(CFG_BUS_CLK) */

	return 0;
}

/*===========================================================================
 * board_early_init_r() - Tsi108 initialization function executed right after
 *                        relocation. Contains code that cannot be executed 
 *                        from flash.
 *==========================================================================*/

int board_early_init_r(void)
{
ulong temp, i;
ulong reg_val;
volatile ulong *reg_ptr;

    /*---------------------------------------------------------------------------
     * After reset the base address for bridge registers is set to 0xC000_0000.
     * Registers for the processor interface block (PB_xx) are located
     * at offset 0x3000 from the base address. 
	 */

    /*---------------------------------------------------------------------------
     * Initialize access through the Switching Fabric (SF) using PB_OCN_BAR2.
     *---------------------------------------------------------------------------
     *
     * This window provides acces to the PCI Prefetchable Memory Space.
     * For purpose of verification of Tsi108 operations it also provides mapping
     * to SDRAM controller through the OCN path.
     * The first 16 LUT entries are mapped to PCI port and other 16 entries - to SDRAM OCN port.
     * Total BAR size is 256MB. LUT entry size is 8MB (0x00800000).
	 */
 
    /*-------------------------------------------------------
     * Map 128MB to PCI port  (first 16 LUT entries) */

    /* initialize pointer to LUT associated with PB_OCN_BAR2 */
    reg_ptr = (ulong *)(CFG_TSI108_CSR_BASE + TSI108_PB_REG_OFFSET + 0x900);

    for (i = 0; i < 16; i++) {
        *reg_ptr++ = 0x00000001;
        *reg_ptr++ = 0x00;
    }

    __asm__ __volatile__ ("eieio");
    __asm__ __volatile__ ("sync");

    /*-------------------------------------------------------
     * Map 128MB to SDRAM OCN port (next 16 LUT entries) */

    reg_val = 0x18000044; /* PB_LOWER_LUT_ADDRx = SDRAM port + Addr_Translation Enabled */
    for (i = 0; i < 16; i++) {
        *reg_ptr++ = reg_val;
        *reg_ptr++ = 0x00;      /* value for PB_UPPER_LUT_ADDRx */
        reg_val += 0x00800000;  /* add translated addr offset value for PB_LOWER_LUT_ADDRx */
                                /* offset = (LUT entry size) */
    }
    __asm__ __volatile__ ("eieio");
    __asm__ __volatile__ ("sync");

    /* Setup PB_OCN_BAR2: size 256B + ENable @ 0x0_80000000 */

    out32(CFG_TSI108_CSR_BASE + TSI108_PB_REG_OFFSET + PB_OCN_BAR2,0x80000001);

    /* Make sure that OCN_BAR2 decoder is set (to allow following immediate read from SDRAM) */
    temp = in32(CFG_TSI108_CSR_BASE + TSI108_PB_REG_OFFSET + PB_OCN_BAR2);


    /*---------------------------------------------------------------------------
     * Remap PB_OCN_BAR1 to accomodate PCI-bus aperture and EPROM into the
     * processor bus address space. Immediately after reset LUT and address
     * translation are disabled for this BAR. Now we have to initialize LUT and
     * switch from the BOOT mode to the normal operation mode.
	 */

    /* The aperture defined by PB_OCN_BAR1 startes at address 0xE0000000
     * and covers 512MB of address space. To allow larger aperture we also have to relocate
     * register window of Tsi108
	 */
 
    /*
	 * Initialize LUT (32-entries) prior switching PB_OCN_BAR1 from BOOT mode
	 */

    /* initialize pointer to LUT associated with PB_OCN_BAR1 */
    reg_ptr = (ulong *)(CFG_TSI108_CSR_BASE + TSI108_PB_REG_OFFSET + 0x800);

    for (i = 0; i < 32; i++) {
        *reg_ptr++ = pb2ocn_lut1[i].lower;
        *reg_ptr++ = pb2ocn_lut1[i].upper;
    }
	__asm__ __volatile__ ("sync");

    /* Set new value for PB_OCN_BAR1: switch from BOOT to LUT mode.
     * value for PB_OCN_BAR1: (BA-0xE000_0000 + size 512MB + ENable)
	 */

    out32(CFG_TSI108_CSR_BASE + TSI108_PB_REG_OFFSET + PB_OCN_BAR1, 0xE0000011);

    /* Make sure that OCN_BAR2 decoder is set (to allow following immediate read from SDRAM) */
    temp = in32(CFG_TSI108_CSR_BASE + TSI108_PB_REG_OFFSET + PB_OCN_BAR1);

#ifndef DISABLE_PBM
    /*
     * For IBM processors we have to set Address-Only commands generated by PBM
     * that are different from ones set by default after reset.
     */
     
    temp = get_cpu_type();

    if ((CPU_750FX == temp) || (CPU_750GX == temp)) {
        out32(CFG_TSI108_CSR_BASE + TSI108_PB_REG_OFFSET + PB_MCMD, 0x00009955);
    }
#endif /* DISABLE_PBM */

#ifdef CONFIG_PCI
    /*--------------------------------------------------------------------------
     * Initialize PCI/X block
     *------------------------------------------------------------------------*/

    /* Map PCI/X Configuration Space (16MB @ 0x0_FE000000) */

    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_PFAB_BAR0_UPPER, 0);
    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_PFAB_BAR0, 0xFE000001);

    /* Set Bus Number for the attached PCI/X bus (we will use 0 for NB) */
    temp = in32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_PCIX_STAT);
    temp &= ~0xFF00; /* Clear the BUS_NUM field */
    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_PCIX_STAT, temp);

    /* Map PCI/X IO Space (64KB @ 0x0_FD000000) takes one 16MB LUT entry */
    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_PFAB_IO_UPPER, 0);
    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_PFAB_IO, 0xFD000001);

    /* Map PCI/X Memory Space */

    /* Transactions directed from OCM to PCI Memory Space are directed from PB to PCI 
     * unchanged (as defined by PB_OCN_BAR1,2 and LUT settings).
     * If address remapping is required the corresponding PCI_PFAB_MEM32
     * and PCI_PFAB_PFMx register groups have to be configured. */

    /*--------------------------------------------------------------------------
     * Map the path from the PCI/X bus into the system memory
     *------------------------------------------------------------------------*/

    /*--------------------------------------------------------------------------
     * The memory mapped window assotiated with PCI P2O_BAR2 provides access to the
     * system memory without address remapping.
     * All system memory is opened for accesses initiated by PCI/X bus masters.
	 */

    /* Initialize LUT associated with PCI P2O_BAR2 */

    /* set pointer to LUT associated with PCI P2O_BAR2 */
    reg_ptr = (ulong *)(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + 0x500);
#ifdef DISABLE_PBM
    /* In case when PBM is disabled (no HW supported cache snoopng on PB) P2O_BAR2 
     * is directly mapped into the system memory without address translation. */

    reg_val = 0x00000004; /* SDRAM port + NO Addr_Translation */
    for (i = 0; i < 32; i++) {
        *reg_ptr++ = reg_val;  /* P2O_BAR2_LUTx */
        *reg_ptr++ = 0;        /* P2O_BAR2_LUT_UPPERx */
    }
    reg_val = 0x00007500; /* value for PCI BAR2 (size = 512MB, Enabled, No Addr. Translation) */
#else
    reg_val = 0x00000002; /* Destination port = PBM */
    for (i = 0; i < 32; i++) {
        *reg_ptr++ = reg_val;    /* P2O_BAR2_LUTx */
        *reg_ptr++ = 0x40000000; /* P2O_BAR2_LUT_UPPERx : Set data swapping mode for PBM (byte swapping) */
        reg_val += 0x01000000;   /* offset = 16MB, address translation is enabled to allow byte swapping */
    }
    reg_val = 0x00007100; /* value for PCI BAR2 (size = 512MB, Enabled, Address Translation Enabled) */
#endif /* DISABLE_PBM */
    __asm__ __volatile__ ("eieio");
    __asm__ __volatile__ ("sync");
    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_P2O_PAGE_SIZES, reg_val);

    /* Set 64-bit PCI bus address for system memory ( 0 is the best choice for easy mapping) */
    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_P2O_BAR2, 0x00000000);
    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_P2O_BAR2_UPPER, 0x00000000);

#ifndef DISABLE_PBM
    /*--------------------------------------------------------------------------
     * The memory mapped window assotiated with PCI P2O_BAR3 provides access to the
     * system memory using SDRAM OCN port and address translation. This is alternative
     * way to access SDRAM from PCI required for Tsi108 emulation testing.
     * All system memory is opened for accesses initiated by PCI/X bus masters.
	 */

    /* Initialize LUT associated with PCI P2O_BAR3 */

    /* set pointer to LUT associated with PCI P2O_BAR3 */
    reg_ptr = (ulong *)(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + 0x600);
    reg_val = 0x00000004; /* Destination port = SDC */
    for (i = 0; i < 32; i++) {
        *reg_ptr++ = reg_val;   /* P2O_BAR3_LUTx */
        *reg_ptr++ = 0;         /* P2O_BAR3_LUT_UPPERx : Set data swapping mode for PBM (byte swapping) */
        reg_val += 0x01000000;  /* offset = 16MB, address translation is enabled to allow byte swapping */
    }
    __asm__ __volatile__ ("eieio");
    __asm__ __volatile__ ("sync");

    /* Configure PCI P2O_BAR3 (size = 512MB, Enabled) */
    reg_val = in32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_P2O_PAGE_SIZES);
    reg_val &= ~0x00FF;
    reg_val |= 0x0071;
    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_P2O_PAGE_SIZES, reg_val);

    /* Set 64-bit base PCI bus address for window (0x20000000) */
    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_P2O_BAR3_UPPER, 0x00000000);
    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_P2O_BAR3, 0x20000000);
#endif /* !DISABLE_PBM */

#ifdef ENABLE_PCI_CSR_BAR /* open if required access to Tsi108 CSRs from the PCI/X bus */
    /* enable BAR0 on the PCI/X bus */
    reg_val = in32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_MISC_CSR);
    reg_val |= 0x02;
    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_MISC_CSR, reg_val);

    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_P2O_BAR0_UPPER, 0x00000000);
    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_P2O_BAR0, CFG_TSI108_CSR_BASE);
#endif

#ifdef PCI_PRE_DRIVE_ENABLED
    /* PCI-mode only: */
    /* If enabled, PCI config address is driven four PCI clocks before assertion */
    /* of FRAME#.  In PCI-X, the address is always pre-driven. */
    temp = in32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_MISC_CSR);
    temp |= PCI_MISC_CSR_PRE_DRIVE;
    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_MISC_CSR, temp);
#endif

#ifdef PCI_32BIT_MODE_ENABLED
    /* 32-bit mode */
    temp = in32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_MISC_CSR);
    temp |= PCI_MISC_CSR_SYS32;
    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_MISC_CSR, temp);
#endif

    /*--------------------------------------------------------------------------
     * Finally enable PCI/X Bus Master and Memory Space access */

    reg_val = in32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_CSR);
    reg_val |= 0x06;
    out32(CFG_TSI108_CSR_BASE + TSI108_PCI_REG_OFFSET + PCI_CSR, reg_val);
#endif /* CONFIG_PCI */

    /*--------------------------------------------------------------------------
     * Initialize MPIC outputs (interrupt pins): 
     * Interrupt routing on the Grendel Emul. Board:
     *    PB_INT[0] -> INT (CPU0)
     *    PB_INT[1] -> INT (CPU1)
     *    PB_INT[2] -> MCP (CPU0)
     *    PB_INT[3] -> MCP (CPU1)
     * Set interrupt controller outputs as Level_Sensitive/Active_Low
	 */

    out32(CFG_TSI108_CSR_BASE + TSI108_MPIC_REG_OFFSET + MPIC_CSR(0), 0x02);
    out32(CFG_TSI108_CSR_BASE + TSI108_MPIC_REG_OFFSET + MPIC_CSR(1), 0x02);
    out32(CFG_TSI108_CSR_BASE + TSI108_MPIC_REG_OFFSET + MPIC_CSR(2), 0x02);
    out32(CFG_TSI108_CSR_BASE + TSI108_MPIC_REG_OFFSET + MPIC_CSR(3), 0x02);

    /*--------------------------------------------------------------------------
     * Ensure that Machine Check exception is enabled
     * We need it to support PCI Bus probing (configuration reads)
	 */

    reg_val = mfmsr();
    mtmsr(reg_val | MSR_ME);

    return 0;
}


/*===========================================================================
 * misc_init_r()
 *            
 * various things to do after relocation
 *            
 *==========================================================================*/

int misc_init_r(void)
{
#ifdef CFG_CLK_SPREAD  /* Initialize Spread-Spectrum Clock generation */
ulong i;

    /* Ensure that Spread-Spectrum is disabled */
    out32(CFG_TSI108_CSR_BASE + TSI108_CLK_REG_OFFSET + CG_PLL0_CTRL0, 0);
    out32(CFG_TSI108_CSR_BASE + TSI108_CLK_REG_OFFSET + CG_PLL1_CTRL0, 0);

    /* Initialize PLL1: CG_PCI_CLK , internal OCN_CLK
     * Uses pre-calculated value for Fout = 800 MHz, Fs = 30 kHz, D = 0.5%
     */

    out32(CFG_TSI108_CSR_BASE +
        TSI108_CLK_REG_OFFSET + CG_PLL1_CTRL1, 0x00000039); /* BWADJ */
    out32(CFG_TSI108_CSR_BASE +
        TSI108_CLK_REG_OFFSET + CG_PLL1_CTRL0, 0x002e0044); /* D = 0.25% */

    /* Initialize PLL0: CG_PB_CLKO  */
    /* Detect PB clock freq. */
    i = in32(CFG_TSI108_CSR_BASE + TSI108_CLK_REG_OFFSET + CG_PWRUP_STATUS);
    i = (i >> 16) & 0x07; /* Get PB PLL multiplier */

    out32(CFG_TSI108_CSR_BASE +
        TSI108_CLK_REG_OFFSET + CG_PLL0_CTRL1, pll0_config[i].ctrl1);
    out32(CFG_TSI108_CSR_BASE +
        TSI108_CLK_REG_OFFSET + CG_PLL0_CTRL0, pll0_config[i].ctrl0);

    /* Wait and set SSEN for both PLL0 and 1 */
	udelay(1000);
    out32(CFG_TSI108_CSR_BASE +
        TSI108_CLK_REG_OFFSET + CG_PLL0_CTRL0, 0x80000000 | pll0_config[i].ctrl0);
    out32(CFG_TSI108_CSR_BASE +
        TSI108_CLK_REG_OFFSET + CG_PLL1_CTRL0, 0x802e0044); /* D=0.25% */
#endif /* CFG_CLK_SPREAD */

#ifdef CFG_L2
	l2cache_enable ();
#endif
    return 0;
}
