/*
 * (C) Copyright 2005 Tundra Semiconductor Corp.
 * Alex Bounine, <alexandreb@tundra.com).
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * tsi108emu.c - board support/init functions for the Tsi108 emulation board.
 */

#include <common.h>
#include <74xx_7xx.h>
#include <command.h>
#include "tsi108.h"

#undef	DEBUG

void after_reloc (ulong dest_addr)
{
	DECLARE_GLOBAL_DATA_PTR;

	/* Jump to the rest of board init code */
	board_init_r ((gd_t *)gd, dest_addr);
	/* NOTREACHED */
}


/*--------------------------------------------------------------------------
 *
 * Check Board Identity:
 *
 * reports board type
 */

int checkboard (void)
{
	DECLARE_GLOBAL_DATA_PTR;
	int l_type = 0;

	printf ("BOARD: %s : Processor Bus @ %d MHz\n",
		CFG_BOARD_NAME, (gd->bus_clk/1000000));
	return (l_type);
}


/*-------------------------------------------------------------------------
 * read_pid()
 *
 * Reports calling processor ID.
 * Stub function: for single-CPU configurations always returns 0
 *-----------------------------------------------------------------------*/

int read_pid(void)
{
	return 0;
}

/*-------------------------------------------------------------------------*/

long int dram_size (int board_type)
{
	 /* We keep rest of all available memory reserved for testing */
	return (CFG_UBOOT_MEM_SIZE * 0x100000);
}

/* ------------------------------------------------------------------------- */

long int initdram (int board_type)
{
	return dram_size (board_type);
}

/* ------------------------------------------------------------------------- */

#if defined (CFG_DRAM_TEST)

/* DRAM check routines to be copied from gw8260 */
/* FIXME!!! Not implemented here yet */

int testdram (void)
{
	printf ("SDRAM Test Stub: ??? MB - ok\n");
	return (0);
}

#endif /* CFG_DRAM_TEST */

static inline void
soft_restart(unsigned long addr)
{
	/* SRR0 has system reset vector, SRR1 has default MSR value */
	/* rfi restores MSR from SRR1 and sets the PC to the SRR0 value */

	__asm__ __volatile__ ("mtspr	26, %0"		:: "r" (addr));
	__asm__ __volatile__ ("li	4, (1 << 6)"	::: "r4");
	__asm__ __volatile__ ("mtspr	27, 4");
	__asm__ __volatile__ ("rfi");

	while(1);	/* not reached */
}
#if (0)
/* no generic way to do board reset. simply call soft_reset. */
void
do_reset (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	ulong addr;
	/* flush and disable I/D cache */
	__asm__ __volatile__ ("mfspr	3, 1008"	::: "r3");
	__asm__ __volatile__ ("ori	5, 5, 0xcc00"	::: "r5");
	__asm__ __volatile__ ("ori	4, 3, 0xc00"	::: "r4");
	__asm__ __volatile__ ("andc	5, 3, 5"	::: "r5");
	__asm__ __volatile__ ("sync");
	__asm__ __volatile__ ("mtspr	1008, 4");
	__asm__ __volatile__ ("isync");
	__asm__ __volatile__ ("sync");
	__asm__ __volatile__ ("mtspr	1008, 5");
	__asm__ __volatile__ ("isync");
	__asm__ __volatile__ ("sync");

#ifdef CFG_RESET_ADDRESS

	addr = CFG_RESET_ADDRESS;

	/*
	 Set PB_OCN_BAR1 BOOT bit.
	
	Setting BOOT bit so that the reset vector will be 
	masked to use only PB_A[16:35] just like during cold boot.
	
	Note with BOOT bit set:
	  1.  all memory regions are write-protected
	  2.  PB_A[16:35] bits are used ie.  0000_xxxx

	Otherwise, with the current memory map, the Intel 16M flash will 
	receive the address 0x0FF0_0100 from HLP which will
	correspond to 0x00F0_0100 (sector 121).

	At cold boot time, the PB_OCN_BAR1[BOOT] bit forces HLP to 
	send address 0x0000_0100, which corresponds to the 
	first sector of the flash.

	The monitor by default is loaded in the first sector of the 
	16M Intel flash.

	*/
	*(ulong*)(CFG_TSI108_CSR_RST_BASE + TSI108_PB_REG_OFFSET + PB_OCN_BAR1) = 
	(*(ulong*)((CFG_TSI108_CSR_RST_BASE + TSI108_PB_REG_OFFSET + PB_OCN_BAR1)))
	| 2;


#else
	/*
	 * note: when CFG_MONITOR_BASE points to a RAM address,
	 * CFG_MONITOR_BASE - sizeof (ulong) is usually a valid
	 * address. Better pick an address known to be invalid on your
	 * system and assign it to CFG_RESET_ADDRESS.
	 */
	addr = CFG_MONITOR_BASE - sizeof (ulong);
#endif
	soft_restart(addr);
	while(1);	/* not reached */
}
#endif

