/*
 * (C) Copyright 2005 Tundra Semiconductor Corp.
 * Alex Bounine, <alexandreb@tundra.com).
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * tsi108evb.h - board specific configuration options for
 *               Tsi108/750GX Evaluation Board
 */

#ifndef __CONFIG_H
#define __CONFIG_H

#include <asm/processor.h>

#undef DEBUG

/* Board Configuration Definitions */

#define CONFIG_TSI108EVB	/* EV board (very close to SVB)*/

#define CONFIG_7xx
#define CONFIG_750FX /* this option to enable init of extended BATs */
#define CONFIG_TSI108

#define CFG_BOARD_NAME		"TSI108EVB"
#define CONFIG_IDENT_STRING	"Tsi108/750GX Evaluation Board (1.0)"
#define CFG_OCN_CLK		133000000    /* 133 MHz */
/*#define CFG_CLK_SPREAD*/  /* Enable Spread-Spectrum Clock generation */

#ifndef CFG_UBOOT_MEM_SIZE
#define CFG_UBOOT_MEM_SIZE	32 /* U-Boot Memory Size Limit (MBytes) */
#endif

#define CFG_TSI108_EXPORTS	/* Use extra export calls */

#undef  CONFIG_ECC		/* disable ECC support */
#undef  DISABLE_PBM		/* PB Master enables HW memory coherency support */

/* Board-specific Initialization Functions to be called */
#define CFG_BOARD_ASM_INIT
#define CONFIG_BOARD_EARLY_INIT_F
#define CONFIG_BOARD_EARLY_INIT_R
#define CONFIG_MISC_INIT_R

/*
 * High Level Configuration Options
 * (easy to change)
 */

#define CONFIG_BAUDRATE		115200	/* console baudrate = 115000 */

/*#define CFG_HUSH_PARSER */
#undef CFG_HUSH_PARSER

#define CFG_PROMPT_HUSH_PS2	"> "

/*
 * The following defines let you select what serial you want to use
 * for your console driver.
 *
 * what to do:
 * If you have hacked a serial cable onto the second DUART channel, change the CFG_DUART port from 1
 * to 0 below.
 *
 */

/*#define CFG_NS16550_SERIAL */
#define CFG_NS16550
#define CFG_NS16550_REG_SIZE    1

#if 1
#define CONFIG_BOOTDELAY	-1	/* autoboot disabled		*/
#else
#define CONFIG_BOOTDELAY	3	/* autoboot after 5 seconds	*/
#endif
#define CONFIG_ZERO_BOOTDELAY_CHECK


#undef	CONFIG_BOOTARGS
/*#define CONFIG_PREBOOT	"echo;echo Type \"run flash_nfs\" to mount root filesystem over NFS;echo" */

#if (CONFIG_BOOTDELAY >= 0)
#define CONFIG_BOOTCOMMAND	"tftpboot 0x400000 zImage.initrd.elf;\
 setenv bootargs $(bootargs) $(bootargs_root) nfsroot=$(serverip):$(rootpath) \
 ip=$(ipaddr):$(serverip)$(bootargs_end);  bootm 0x400000; "

#define CONFIG_BOOTARGS "console=ttyS0,115200"
#endif

#undef CONFIG_EXTRA_ENV_SETTINGS

/* Networking Configuration */

#define CONFIG_TSI108_GIGE
#define TSI108_PHY0_ADDR	(1)	/* PHY address for GIGE port 0 */
#define TSI108_PHY1_ADDR	(2)	/* PHY address for GIGE port 1 */

/* Default MAC Addresses for on-chip GIGE Controller */
#define CONFIG_ETHADDR				00:06:D2:00:00:90
#define CONFIG_ETH1ADDR				00:06:D2:00:00:91
//#define CONFIG_HAS_ETH1       (1)
#define CONFIG_ENV_OVERWRITE    

/* to change the default ethernet port, use this define (options: 0, 1, 2) */
#define CONFIG_NET_MULTI      //(1)

#define CONFIG_IPADDR		10.100.10.12
#define CONFIG_SERVERIP		10.100.10.20
#define CONFIG_NETMASK      255.255.255.0

#define CONFIG_BOOTFILE     zImage.elf
#define CONFIG_LOADADDR     0x400000

/*#define CONFIG_ROOTPATH /opt/eldk/ppc_74xx */

#define CONFIG_TESTDRAMDATA	    y
#define CONFIG_TESTDRAMADDRESS	n
#define CONFIG_TESETDRAMWALK	n

/* --------------------------------------------------------------------------------------------------------------- */

#define CONFIG_LOADS_ECHO	0	/* echo off for serial download */
#define CFG_LOADS_BAUD_CHANGE		/* allow baudrate changes	*/

#undef	CONFIG_WATCHDOG			/* watchdog disabled		*/
#undef	CONFIG_ALTIVEC			/* undef to disable		*/

#define CONFIG_BOOTP_MASK	(CONFIG_BOOTP_DEFAULT | \
				 CONFIG_BOOTP_BOOTFILESIZE)

/* Flash banks JFFS2 should use */
#define CFG_JFFS2_FIRST_BANK	1
#define CFG_JFFS2_NUM_BANKS	1

#define CONFIG_COMMANDS ((CONFIG_CMD_DFL \
			| CFG_CMD_ASKENV \
			| CFG_CMD_CACHE \
			| CFG_CMD_JFFS2 \
			| CFG_CMD_KGDB \
			| CFG_CMD_PCI \
			| CFG_CMD_I2C \
			| CFG_CMD_BG \
			| CFG_CMD_SDRAM \
			| CFG_CMD_EEPROM \
			| CFG_CMD_PING \
			| CFG_CMD_DATE\
			| CFG_CMD_NET ))



/* this must be included AFTER the definition of CONFIG_COMMANDS (if any) */
#include <cmd_confdefs.h>

/*
 * Miscellaneous configurable options
 */
#define CONFIG_RTC_DS1337		/* Use DS1337 RTC via i2c */
#define CFG_I2C_RTC_ADDR	(0x68)	/* @ address 0x68 */

#define CFG_I2C_EEPROM_ADDR		0x50		/* I2C EEPROM page 1 */
#define CFG_I2C_EEPROM_ADDR_LEN		1		/* Bytes of address		*/
#define CFG_I2C_EEPROM_WRITE_CYCLE_TIME  (6000)  /* Units: microseconds      */
                                                 /* Note:  24LC16B needs 5ms */
						 /*        udelay() is not   */
						 /*        accurate          */
/*#define CFG_I2C_MULTI_EEPROMS */
#define CFG_I2C_SPEED	100000		/* I2C speed default - 100 kHz */

#define CFG_LONGHELP			/* undef to save memory		*/
#define CFG_PROMPT	"=>"		/* Monitor Command Prompt	*/

#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
//#define CFG_KGDB_USE_OTHER_PORT     /* Use KGDB communication via other port */

#define CFG_CBSIZE	1024		/* Console I/O Buffer Size	*/
#define CONFIG_KGDB_BAUDRATE	115200	/* speed to run kgdb serial port at */
#else
#define CFG_CBSIZE	256		/* Console I/O Buffer Size	*/
#endif

#define CFG_PBSIZE (CFG_CBSIZE+sizeof(CFG_PROMPT)+16) /* Print Buffer Size */
#define CFG_MAXARGS	16		/* max number of command args	*/
#define CFG_BARGSIZE	CFG_CBSIZE	/* Boot Argument Buffer Size	*/

/*
#define CFG_DRAM_TEST
 * DRAM tests
 *   CFG_DRAM_TEST - enables the following tests.
 *
 *   CFG_DRAM_TEST_DATA - Enables test for shorted or open data lines
 *			  Environment variable 'test_dram_data' must be
 *			  set to 'y'.
 *   CFG_DRAM_TEST_DATA - Enables test to verify that each word is uniquely
 *			  addressable. Environment variable
 *			  'test_dram_address' must be set to 'y'.
 *   CFG_DRAM_TEST_WALK - Enables test a 64-bit walking ones pattern test.
 *			  This test takes about 6 minutes to test 64 MB.
 *			  Environment variable 'test_dram_walk' must be
 *			  set to 'y'.
 */
#define CFG_DRAM_TEST
#if defined(CFG_DRAM_TEST)
#define CFG_MEMTEST_START		0x00400000	/* memtest works on	*/
#define CFG_MEMTEST_END		    0x07c00000	/* 4 ... 124 MB in DRAM */
/*#define CFG_DRAM_TEST_DATA */
/*#define CFG_DRAM_TEST_ADDRESS */
/*#define CFG_DRAM_TEST_WALK */
#endif /* CFG_DRAM_TEST */

#define CFG_LOAD_ADDR		0x00400000	/* default load address */

#define CFG_HZ			1000		/* !!! FIX_ME decr freq: 1ms ticks */


/*
 * Low Level Configuration Settings
 * (address mappings, register initial values, etc.)
 * You should know what you are doing if you make changes here.
 */

/*-----------------------------------------------------------------------
 * Definitions for initial stack pointer and data area
 */

/*
 * When locking data in cache you should point the CFG_INIT_RAM_ADDRESS
 * To an unused memory region. The stack will remain in cache until RAM
 * is initialized
*/
#undef  CFG_INIT_RAM_LOCK
#define CFG_INIT_RAM_ADDR	(CFG_UBOOT_MEM_SIZE << 20) /* set above U-Boot space limit */
#define CFG_INIT_RAM_END	0x4000     /* larger space - we have SDRAM initialized */

#define CFG_GBL_DATA_SIZE	128  /* size in bytes reserved for init data */
#define CFG_GBL_DATA_OFFSET	(CFG_INIT_RAM_END - CFG_GBL_DATA_SIZE)

/*-----------------------------------------------------------------------
 * Start addresses for the final memory configuration
 * (Set up by the startup code)
 * Please note that CFG_SDRAM_BASE _must_ start at 0
 */

#define CFG_SDRAM_BASE		0x00000000  /* first 256 MB of SDRAM */
#define CFG_SDRAM1_BASE		0x10000000	/* next 256MB of SDRAM */
#define CFG_SDRAM2_BASE		0x40000000  /* beginning of non-cacheable alias for SDRAM - first 256MB */
#define CFG_SDRAM3_BASE		0x50000000  /* next Non-Cacheable 256MB of SDRAM */
#define CFG_PCI_PFM_BASE	0x80000000  /* Prefetchable (cacheable) PCI/X PFM and SDRAM OCN (128MB+128MB) */
#define CFG_PCI_MEM32_BASE	0xE0000000  /* Non-Cacheable PCI/X MEM and SDRAM OCN (128MB+128MB) */
#define CFG_MISC_REGION_BASE  0xf0000000  /* Base Address for (PCI/X + Flash) region */
#define CFG_FLASH_BASE		0xfff00000  /* Base Address of Flash device */

#define CONFIG_VERY_BIG_RAM	    /* we will use up to 256M memory for cause we are short of BATS*/
#define CFG_RESET_ADDRESS		0xfff00100
#define CFG_MONITOR_LEN		(256 << 10)	/* Reserve 256 kB for Monitor */
#define CFG_MONITOR_BASE		CFG_FLASH_BASE
#define CFG_MALLOC_LEN		(512 << 10)	/* Reserve 256 kB for malloc */


/* Peripheral Device section */

/*******************************************************/
/* Resources on the Tsi108 EMU Board :		           */
/*******************************************************/

#define CFG_TSI108_CSR_RST_BASE 0xC0000000                  /* Tsi108 CSR base after reset */
#define CFG_TSI108_CSR_BASE     CFG_TSI108_CSR_RST_BASE     /* Runtime Tsi108 CSR base */
#define ENABLE_PCI_CSR_BAR /* enables access to Tsi108 CSRs from the PCI/X bus */

#define CFG_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }
#define CFG_DUART_IO		(CFG_TSI108_CSR_BASE + 0x7808)

#define CFG_DUART_CHAN	0			/* channel to use for console */

/*-----------------------------------------------------------------------
 * PCI stuff
 *-----------------------------------------------------------------------
 */

#define CONFIG_PCI			/* include pci support		*/

#define PCI_HOST_ADAPTER 0		/* configure as pci adapter	*/
#define PCI_HOST_FORCE	1		/* configure as pci host	*/
#define PCI_HOST_AUTO	2		/* detected via arbiter enable	*/
#define CONFIG_PCI_HOST PCI_HOST_FORCE	/* select pci host function	*/
#define CONFIG_PCI_PNP			/* do pci plug-and-play		*/

/* PCI MEMORY MAP section */

/* PCI view of System Memory */
#define CFG_PCI_MEMORY_BUS      0x00000000
#define CFG_PCI_MEMORY_PHYS     0x00000000
#define CFG_PCI_MEMORY_SIZE     0x20000000

/* PCI Memory Space */
#define CFG_PCI_MEM_BUS         (CFG_PCI_MEM_PHYS)
#define CFG_PCI_MEM_PHYS        (CFG_PCI_MEM32_BASE)
#define CFG_PCI_MEM_SIZE        0x08000000

/* PCI I/O Space */
#define CFG_PCI_IO_BUS          0x00000000
#define CFG_PCI_IO_PHYS         0xfd000000
#define CFG_PCI_IO_SIZE         0x00010000
#define _IO_BASE				0x00000000	/* points to PCI I/O space */

/* PCI Config Space mappig */
#define CFG_PCI_CFG_BASE	0xFE000000
#define CFG_PCI_CFG_SIZE	0x01000000  /* 16MB */

/*----------------------------------------------------------------------
 * Initial BAT mappings
 */

/* NOTES:
 * 1) GUARDED and WRITE_THRU not allowed in IBATS
 * 2) CACHEINHIBIT and WRITETHROUGH not allowed together in same BAT
 */

/* IBAT0 - FLASH ICache enabled */
#define CFG_IBAT0L (CFG_FLASH_BASE | BATL_PP_RW )
#define CFG_IBAT0U (CFG_FLASH_BASE | BATU_BL_1M | BATU_VS | BATU_VP)

/* DBAT0 - PCI/X(MEM N/C) + PCI/X(I/O) + PCI/X(CFG) + FLASH */
#define CFG_DBAT0L (CFG_MISC_REGION_BASE | BATL_PP_RW | BATL_CACHEINHIBIT | BATL_GUARDEDSTORAGE)
#define CFG_DBAT0U (CFG_MISC_REGION_BASE | BATU_BL_256M | BATU_VS | BATU_VP)

/* IBAT1/DBAT1 - SDRAM (first 256MB) */
#define CFG_IBAT1L (CFG_SDRAM_BASE | BATL_PP_RW | BATL_MEMCOHERENCE)
#define CFG_IBAT1U (CFG_SDRAM_BASE | BATU_BL_256M | BATU_VS | BATU_VP)
#define CFG_DBAT1L (CFG_SDRAM_BASE | BATL_PP_RW | BATL_MEMCOHERENCE)
#define CFG_DBAT1U (CFG_SDRAM_BASE | BATU_BL_256M | BATU_VS | BATU_VP)

/* IBAT2/DBAT2 - SDRAM (second 256MB) */
#define CFG_IBAT2L (CFG_SDRAM1_BASE | BATL_PP_RW | BATL_MEMCOHERENCE)
#define CFG_IBAT2U (CFG_SDRAM1_BASE | BATU_BL_256M | BATU_VS | BATU_VP)
#define CFG_DBAT2L (CFG_SDRAM1_BASE | BATL_PP_RW | BATL_MEMCOHERENCE)
#define CFG_DBAT2U (CFG_SDRAM1_BASE | BATU_BL_256M | BATU_VS | BATU_VP)

/* Tsi108 CSR */
#define CFG_IBAT3L BATL_NO_ACCESS
#define CFG_IBAT3U BATL_NO_ACCESS
#define CFG_DBAT3L (CFG_TSI108_CSR_BASE | BATL_CACHEINHIBIT | BATL_PP_RW | BATL_GUARDEDSTORAGE)
#define CFG_DBAT3U (CFG_TSI108_CSR_BASE | BATU_BL_128K | BATU_VS | BATU_VP)

#ifdef CONFIG_750FX

/* IBAT4/DBAT4 - non-cacheable SDRAM alias (first 256MB) */
#define CFG_IBAT4L (CFG_SDRAM2_BASE | BATL_PP_RW | BATL_CACHEINHIBIT)
#define CFG_IBAT4U (CFG_SDRAM2_BASE | BATU_BL_256M | BATU_VS | BATU_VP)
#define CFG_DBAT4L (CFG_SDRAM2_BASE | BATL_PP_RW | BATL_CACHEINHIBIT)
#define CFG_DBAT4U (CFG_SDRAM2_BASE | BATU_BL_256M | BATU_VS | BATU_VP)

/* IBAT5/DBAT5 - non-cacheable SDRAM alias (second 256MB) */
#define CFG_IBAT5L (CFG_SDRAM3_BASE | BATL_PP_RW | BATL_CACHEINHIBIT)
#define CFG_IBAT5U (CFG_SDRAM3_BASE | BATU_BL_256M | BATU_VS | BATU_VP)
#define CFG_DBAT5L (CFG_SDRAM3_BASE | BATL_PP_RW | BATL_CACHEINHIBIT)
#define CFG_DBAT5U (CFG_SDRAM3_BASE | BATU_BL_256M | BATU_VS | BATU_VP)

/* IBAT6/DBAT6 - Non-Cacheable PCI/X MEM and SDRAM OCN (128MB+128MB) */
#define CFG_IBAT6L BATL_NO_ACCESS
#define CFG_IBAT6U BATL_NO_ACCESS
#define CFG_DBAT6L (CFG_PCI_MEM32_BASE | BATL_PP_RW | BATL_CACHEINHIBIT | BATL_GUARDEDSTORAGE)
#define CFG_DBAT6U (CFG_PCI_MEM32_BASE | BATU_BL_256M | BATU_VS | BATU_VP)

/* IBAT7/DBAT7 - Cacheable (Prefetchable) PCI/X PFM and SDRAM OCN (128MB+128MB) */
#define CFG_IBAT7L BATL_NO_ACCESS
#define CFG_IBAT7U BATL_NO_ACCESS
#define CFG_DBAT7L (CFG_PCI_PFM_BASE | BATL_PP_RW | BATL_MEMCOHERENCE)
#define CFG_DBAT7U (CFG_PCI_PFM_BASE | BATU_BL_256M | BATU_VS | BATU_VP)

#endif /* CONFIG_750FX */


/* I2C addresses for the two DIMM SPD chips */
#define DIMM0_I2C_ADDR	0x50
#define DIMM1_I2C_ADDR	0x51

/*
 * For booting Linux, the board info and command line data
 * have to be in the first 8 MB of memory, since this is
 * the maximum mapped by the Linux kernel during initialization.
 */
#define CFG_BOOTMAPSZ		(8<<20) /* Initial Memory map for Linux */

/*-----------------------------------------------------------------------
 * FLASH organization
 */

/*#define CFG_NO_FLASH */

#ifndef CFG_NO_FLASH
#define CFG_FLASH_PROTECTION  1
#define CFG_MAX_FLASH_BANKS   1	/* max number of memory banks	*/
#define CFG_MAX_FLASH_SECT	128	/* max number of sectors on one chip */

#define CFG_FLASH_ERASE_TOUT	120000	/* Timeout for Flash Erase (in ms) */
#define CFG_FLASH_WRITE_TOUT	500	    /* Timeout for Flash Write (in ms) */
/*#define CFG_FLASH_LOCK_TOUT	500	 Timeout for Flash Lock (in ms) */
#define CFG_FLASH_CFI		1 

#endif /* CFG_NO_FLASH */


/* #define CFG_ENV_IS_NOWHERE */
#define CFG_ENV_IS_IN_EEPROM 
#define CFG_ENV_OFFSET		0x00000200	/* Offset of Environment Sector */
#define CFG_ENV_SIZE		0x00000400	/* Total Size of Environment Space */

/*-----------------------------------------------------------------------
 * Cache Configuration
 */
#define CFG_CACHELINE_SIZE	32	/* For all MPC74xx CPUs		 */
#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CFG_CACHELINE_SHIFT	5	/* log base 2 of the above value */
#endif

/*-----------------------------------------------------------------------
 * L2CR setup -- make sure this is right for your board!
 * look in include/mpc74xx.h for the defines used here
 */

/*#define CFG_L2 */
#undef CFG_L2


#if defined (CONFIG_750CX) || defined (CONFIG_750FX)
#define L2_INIT 0
#else

#define L2_INIT		0
/*
#define L2_INIT		(L2CR_L2SIZ_2M | L2CR_L2CLK_3 | L2CR_L2RAM_BURST | \
			L2CR_L2OH_5 | L2CR_L2CTL | L2CR_L2WT)
*/
#endif

#define L2_ENABLE	(L2_INIT | L2CR_L2E)

/*
 * Internal Definitions
 *
 * Boot Flags
 */
#define BOOTFLAG_COLD	0x01		/* Normal Power-On: Boot from FLASH */
#define BOOTFLAG_WARM	0x02		/* Software reboot		    */

/*
 *  Use custom machine check exception 
 *  *** Note: Enabling will clobber KGDB ***
 */
//#define CFG_TSI_USER_DEFINED_MACHINE_CHECK
#endif	/* __CONFIG_H */
